@@ -7,7 +7,7 @@
 
 from __future__ import print_function
 
-__version__ = '0.5.0'
+__version__ = '0.5.2'
 
 import argparse
 import base64
@@ -60,7 +60,7 @@ class FileBaseHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 			fileName = self.fileName
 		if unquote(self.path) != "/" + fileName:
 			self.send_response(302)
-			self.send_header('Location', '/' + fileName)
+			self.send_header('Location', '/' + quote(fileName))
 			self.end_headers()
 			return True
 		return False
@@ -542,7 +542,9 @@ class DirListingHandler(FileBaseHandler):
 		self.send_header("Content-Length", str(len(listing)))
 		self.send_header('Connection', 'close')
 		self.end_headers()
-		self.wfile.write(listing.encode())
+		if sys.version_info.major >= 3:
+			listing = listing.encode()
+		self.wfile.write(listing)
 
 	def convertSize(self, size):
 		for ext in "KMGT":
@@ -635,7 +637,7 @@ class FilePutter(BaseHTTPServer.BaseHTTPRequestHandler):
 		http://host:8080/testfile will cause the file to be named testfile. If
 		no filename is given, a random name will be generated.
 
-		Files can be uploaded with e.g. curl -X POST -d @file <url> .
+		Files can be uploaded with e.g. curl -T file <url> .
 		"""
 		length = self.getContentLength()
 		if length < 0:
@@ -652,11 +654,11 @@ class FilePutter(BaseHTTPServer.BaseHTTPRequestHandler):
 			return
 
 		# Sometimes clients want to be told to continue with their transfer
-		if self.headers.getheader("Expect") == "100-continue":
+		if self.headers.get("Expect") == "100-continue":
 			self.send_response(100)
 			self.end_headers()
 
-		target = open(cleanFileName, "w")
+		target = open(cleanFileName, "wb")
 		bytesLeft = int(self.headers['Content-Length'])
 		while bytesLeft > 0:
 			bytesToRead = min(self.blockSize, bytesLeft)
@@ -714,7 +716,8 @@ class FilePutter(BaseHTTPServer.BaseHTTPRequestHandler):
 
 class ThreadedHTTPServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
 	def handle_error(self, request, client_address):
-		print("%s ABORTED transmission (Reason: %s)" % (client_address[0], sys.exc_value))
+		_, exc_value, _ = sys.exc_info()
+		print("%s ABORTED transmission (Reason: %s)" % (client_address[0], exc_value))
 
 
 def catchSSLErrors(BaseSSLClass):
@@ -857,7 +860,7 @@ class ServeFile():
 		return None
 
 	def setSSLKeys(self, cert, key):
-		""" Set SSL cert/key. Can be either path to file or pyssl X509/PKey object. """
+		""" Set SSL cert/key. Can be either path to file or pyopenssl X509/PKey object. """
 		self.cert = cert
 		self.key = key
 
@@ -1166,7 +1169,7 @@ def main():
 			sys.exit(1)
 
 	if args.ssl and not HAVE_SSL:
-		print("Error: SSL is not available, please install pyssl (python-openssl).")
+		print("Error: SSL is not available, please install pyopenssl (python3-openssl).")
 		sys.exit(1)
 
 	if args.cert and not args.key:
